CREATE OR REPLACE PACKAGE BODY SAMPLE_PACKAGE 
/************************************************************************/
/*         PACKAGE名 ：SAMPLE_PACKAGE                                   */
/*          処理内容 ：ユーティリティ関数パッケージ                     */
/************************************************************************/
IS

    FUNCTION ISEXISTS(p_empno NUMBER) RETURN BOOLEAN
    /********************************************************************/
    /*       PROCEDURE名 ：ISEXISTS                                     */
    /*          処理内容 ：指定した番号の社員が存在するか調べる         */
    /*              引数 ：従業員コード                                 */
    /********************************************************************/
    IS
        cnt NUMBER;
    BEGIN
        -- 件数を取得
        SELECT COUNT(*) INTO cnt FROM EMP WHERE EMPNO = p_empno;

        -- 1件以上ある場合はTRUEを返す
        RETURN cnt > 0;

    END ISEXISTS;

    PROCEDURE DELETE_EMP(p_empno NUMBER)
    /********************************************************************/
    /*       PROCEDURE名 ：DELETE_EMP                                   */
    /*          処理内容 ：指定した日付以前に入社した社員情報を削除する */
    /*              引数 ：従業員コード                                 */
    /********************************************************************/
    IS
    BEGIN
        -- 指定した従業員コードが存在するか調べる
        IF ISEXISTS(p_empno) THEN
            -- 削除する
            CASCADE_DELETE(p_empno);
        END IF;

    END DELETE_EMP;

    /********************************************************************/
    /*       PROCEDURE名 ：CASCADE_DELETE                               */
    /*          処理内容 ：指定した番号の社員情報を削除する             */
    /*              引数 ：従業員コード                                 */
    /********************************************************************/
    PROCEDURE CASCADE_DELETE(p_empno NUMBER)
    IS
    BEGIN
        --BONUS表から削除する
        DELETE FROM BONUS 
         WHERE ENAME = (SELECT ENAME FROM EMP WHERE EMPNO = p_empno);

        --EMP表から削除する
        DELETE FROM EMP
         WHERE EMPNO = p_empno;

    END CASCADE_DELETE;


    FUNCTION GET_EMP(p_date DATE) RETURN ref_cur
    /********************************************************************/
    /*       PROCEDURE名 ：GET_EMP                                      */
    /*          処理内容 ：指定した日付以降に入社した社員情報を表で返す */
    /*              引数 ：従業員コード                                 */
    /********************************************************************/
    IS
        ret ref_cur;
        s   VARCHAR2(255);
    BEGIN
        -- SQLを実行
        s :=      'SELECT E.EMPNO, E.ENAME, E.JOB, E.SAL, D.DEPTNO, D.DNAME';
        s := s || '  FROM EMP E, DEPT D';
        s := s || ' WHERE E.HIREDATE >= :hiredate';
        s := s || '   AND E.DEPTNO = D.DEPTNO';
        OPEN ret FOR s USING p_date;

        --SQL結果を返す
        RETURN ret;

    END GET_EMP;

END SAMPLE_PACKAGE;
;
